"""Manipulate, verify and de/serialize Asset Administration Shells v3.1."""

# Synchronize with __init__.py and changelog.rst!
__version__ = "1.0.0"
__author__ = "Marko Ristin"
__copyright__ = "2025 Contributors to aas-core3.1-python"
__license__ = "MIT"
__status__ = "Production/Stable"
